clear
clear matrix
clear mata

use "...\master_data.dta", clear

*drop _merge

* USE ONLY TREATMENT *


// uncomment the line below to reproduce Table C2, Column 1
// keep if treat == 0 & ((courses == 3 & disability == 3)|(alcoholic == 3 & disability == 3)|(alcoholic == 3 & courses == 3))

// uncomment the line below to reproduce Table C2, Column 3
// keep if treat == 0

gen last14round = (period >= 14)

set more off

rename subject i
rename period t

sort i t
 
recast int i t donation male treatment alcoholic courses disability last14round

rename donation y
rename treatment treat

gen double pd0=.
label variable pd0 "posterior probability selfish"
gen double pp0=.
gen double pp1=.
gen double pd=.

xtset i t

by i: generate int first=1 if _n==1
by i: generate int last=1 if _n==_N

by i: generate double sum_y=sum(y) 
replace sum_y=. if last~=1

egen double y_max=max(y)

mat p=[3]
mdraws if first==1, neq(1) dr(32) prefix(h) primes(p)

scalar draws=r(n_draws)

recast double h1*

local hlist h1*

quietly{

foreach v of varlist `hlist' {
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

*end of loop


program drop _all
program define panel_hurdle

* SPECIFY ARGUMENTS

	args todo b logl
	tempvar za xb pp0 p1 pp1 ppp1 pp ppp pd ppd 
	tempname s_u s_e rr w
	local hlist h1*

* EXTRACT ELEMENTS OF PARAMETER VECTOR b		
		
	mleval `za' = `b', eq(1) 
	mleval `xb' = `b', eq(2)
	mleval `s_u' = `b', eq(3) scalar
	mleval `s_e' = `b', eq(4) scalar
	mleval `rr' = `b', eq(5) scalar
	mleval `w'=`b', eq(6) scalar
	
* INITIALISE TEMPORARY VARIABLES
		
	quietly gen double `p1'=.
	quietly gen double `pp1'=.
	quietly gen double `ppp1'=0
	quietly gen double `pp0'=.
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	quietly gen double `pd'=.
	quietly gen double `ppd'=0
		
* LOOP FOR EVALUATION OF SUM (OVER r) OF PRODUCT (OVER t)
* ppp IS FOR LIKELIHOOD FUNCTION; ppd IS FOR p(d=1) (USED TO COMPUTE 
* POSTERIOR PROBABILITY OF d=0)
		
	quietly{
		
		foreach v of varlist `hlist' {
	    replace  `p1'= (1-`w')*(1/`s_e')*normalden((y-(`xb' + `s_u'*`v'))/`s_e')+`w'/y_max if y>0
		replace  `p1'=(1-`w')*(1-normal((`xb' + `s_u'*`v')/`s_e')) if y==0
		by i: replace `pp1' = exp(sum(ln(`p1')))
		replace `pp1'=. if last~=1
		replace `ppp1'=`ppp1'+`pp1'
				
		replace `pd'=normal((`za'+`rr'*`v')/sqrt(1-`rr'^2))
		replace `pd'=. if last~=1
				
		replace `pp0'= ((`w'/y_max)^n_pos)*((1-`w')^n_zero)
		replace `pp0'=. if last~=1
				
		replace `pp'=`pd'*`pp1'+(1-`pd')*`pp0'
		replace `ppp'=`ppp'+`pp'
		
		replace `ppd'=`ppd'+`pd'
		}	
* END OF LOOP
		
* DIVISION BY R TO GENERATE REQUIRED AVERAGES (OVER r)
* COMPUTE POSTERIOR PROBABILITY OF d=0 (pd0) AND SEND THIS TO MATA 
		
		replace `ppp'=`ppp'/draws
		
		replace `ppp1'=`ppp1'/draws
		replace `ppd'=`ppd'/draws
				
		replace pd0=`pp0'*(1-`ppd')/(`pp0'*(1-`ppd')+`ppp1'*`ppd')
		putmata pd0, replace
		
* MLSUM COMMAND TO SPECIFY PER-SUBJECT LOG-LIKELIHOOD CONTRIBUTION	
		
		mlsum `logl'=ln(`ppp') if last==1
						
		}
end

* "end" SIGNIFIES END OF LIKELIHOOD EVALUATION PROGRAM "panel_hurdle"


* CREATE VARIABLE LISTS

local listd "  "
local listy "  ib3.alcoholic ib3.courses ib3.disability last14round "


* GENERATE BINARY d INDICATING AT LEAST ONE POSITIVE CONTRIBUTION BY SUBJECT; 
* GENERATE NUMBER OF POSITIVE CONTRIBUTIONS BY SUBJECT (n_pos) AND NUMBER OF 
* ZERO CONTRIBUTIONS (n_zero)

quietly{

by i: replace sum_y=sum(y) 
by i: replace sum_y=sum_y[_N]

gen int d=sum_y>0

gen int y_pos=y>0
by i: gen int n_pos = sum(y_pos)
by i: replace n_pos = n_pos[_N]

gen int y_zero=y==0
by i: gen int n_zero = sum(y_zero)
by i: replace n_zero = n_zero[_N]

}

* OBTAIN STARTING VALUES USING PROBIT AND XTTOBIT.  
* STORE VARIANCE ESTIMATES FROM XTTOBIT

probit d   `listd'      if last==1
mat bprobit=e(b)

xttobit y  `listy'  if  d==1, ll(0)   
mat bxttobit=e(b)

scalar sig_u=_b[sigma_u:_cons]
scalar sig_e=_b[sigma_e:_cons]

* CREATE STARTING VALUE VECTOR

mat start=bprobit, bxttobit,0,0

* SPECIFY CONSTRAINTS ON PARAMETERS
   
constraint 3 [sig_u]_b[_cons]=sig_u  
constraint 4 [sig_e]_b[_cons]=sig_e      
constraint 5 [r]_b[_cons]=0.0
constraint 6 [w]_b[_cons]=0

* ESTIMATE PANEL HURDLE MODEL WITH sig_u, sig_e, r, w ALL CONSTRAINED

ml model d0 panel_hurdle (= `listd') ( = `listy' ) /sig_u /sig_e  /r /w, ///
 constraints( 3 4 5 6)
ml init start, copy
ml max, search(norescale)

* STORE ESTIMATES AS STARTING VECTOR FOR NEXT ESTIMATION

mat start=e(b)

* ESTIMATE PANEL HURDLE MODEL WITH r AND w CONSTRAINED

ml model d0 panel_hurdle (= `listd' ) ( = `listy') /sig_u /sig_e  /r /w, ///
 constraints( 5 6)
ml init start, copy
ml max, search(norescale)

* STORE ESTIMATES AS STARTING VECTOR FOR NEXT ESTIMATION

mat start=e(b)

/* ESTIMATE PANEL HURDLE MODEL WITH ONLY w CONSTRAINED (SET STARTING VALUE FOR r)

mat start[1,`=colnumb(start, "r:_cons")']=-0.1

ml model d0 panel_hurdle (= `listd' ) ( = `listy') /sig_u /sig_e  /r /w, ///
 constraints( 6)
ml init start, copy
ml max, trace  search(norescale)

* STORE ESTIMATES AS STARTING VECTOR FOR NEXT ESTIMATION

mat start=e(b)

*/

/* ESTIMATE PANEL HURDLE MODEL WITH NO CONSTRAINTS (SET STARTING VALUE FOR w)

mat start[1,`=colnumb(start, "w:_cons")']=0.1

ml model d0 panel_hurdle (= `listd' ) ( = `listy') /sig_u /sig_e  /r /w, ///
 constraints( 5 7)
ml init start, copy
ml max, trace search(norescale)

*/

test 2.alcoholic = -1.alcoholic
test 1.courses = -2.courses
test 1.disability = -2.disability
test 2.alcoholic = 1.courses
test 2.alcoholic = 1.disability
test 1.courses = 1.disability


